@@ -117,13 +117,6 @@ gst_rotate_get_property (GObject * object, guint prop_id,
   }
 }
 
-/* Clean up */
-static void
-gst_rotate_finalize (GObject * obj)
-{
-  G_OBJECT_CLASS (parent_class)->finalize (obj);
-}
-
 static gboolean
 rotate_map (GstGeometricTransform * gt, gint x, gint y, gdouble * in_x,
     gdouble * in_y)
@@ -183,21 +176,18 @@ gst_rotate_class_init (GstRotateClass * klass)
   gstelement_class = (GstElementClass *) klass;
   gstgt_class = (GstGeometricTransformClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
-
   gst_element_class_set_static_metadata (gstelement_class,
       "rotate",
       "Transform/Effect/Video",
       "Rotates the picture by an arbitrary angle",
       "Thiago Santos<thiago.sousa.santos@collabora.co.uk>");
 
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_rotate_finalize);
-  gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_rotate_set_property);
-  gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_rotate_get_property);
+  gobject_class->set_property = gst_rotate_set_property;
+  gobject_class->get_property = gst_rotate_get_property;
 
   g_object_class_install_property (gobject_class, PROP_ANGLE,
       g_param_spec_double ("angle", "angle",
-          "Angle at which the arc starts in radians",
+          "Angle by which the picture is rotated, in radians",
           -G_MAXDOUBLE, G_MAXDOUBLE, DEFAULT_ANGLE,
           GST_PARAM_CONTROLLABLE | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
